#ifndef QGISMTWINDOW_H
#define QGISMTWINDOW_H

//Qt Include
#include <QDialog>
#include <QTransform>
#include <QPushButton>
#include <QPoint>
#include <QMenu>
#include <QMainWindow>

//QGis Include
#include <qgisplugin.h>
#include <qgsmaplayer.h>
#include <qgsvectorlayer.h>
#include <qgsrasterlayer.h>
#include <qgsmapcanvas.h>
#include <qgslegendinterface.h>
#include <qgsmaptool.h>
#include <qgsmaptoolpan.h>

//Local Include
#include "fctechnique.h"
#include "fingercount.h"
#include "blobfingercount.h"
#include "qgismtlegend.h"
#include "qgismtglasslayer.h"
#include "qgismttool.h"
#include "qgismttoolpan.h"
#include "qgismttoolselect.h"
#include "qgismttoolzoom.h"
#include "qgismttoolvoid.h"

class QGisMTWindow : public QMainWindow
{
    Q_OBJECT
    
    /** \enum QGisMTWindow::MODE_CMD_CTRL
     * List the different mode the application can be in 
     */
    enum MODE_CMD_CTRL
    {
        ZOOM,
        PITCH,
        PAN,
        SELECT,
        NONE
    };

public:
    /** Constructor need an instance of the QGisInterface*/
    QGisMTWindow ( QgisInterface *iface,QWidget* parent = 0 );

    /** Destructor*/
    virtual ~QGisMTWindow();

    /** 
     * Provide acces to the current MapCanvas
     * \return the pointer to the canvas.
     */
    QgsMapCanvas* getCanvas() {return my_canvas;}

    /** Return the application GlassLayer */
    QGisMTGlassLayer* getGlassLayer() {return glassLayer;}

    /** Calculate distance between 2 QPoint*/
    float dist ( const QPoint& p1, const QPoint& p2 );

    /*! Return the current active layer.
    If no layer is active 0 is returned. */
    virtual QgsMapLayer* getActiveLayer();

    /*! Return the current active Vector layer.
    If the active layer is not of vector type 0 is returned. */
    virtual QgsVectorLayer* getActiveVectorLayer();

    /*! Return the current active Raster layer.
    If the active layer is not of raster type 0 is returned. */
    virtual QgsRasterLayer* getActiveRasterLayer();

protected:
    /** the Edit menu */
    QMenu* editMenu;

    /** An empty action*/
    QAction* actionUnknownGeom;

    /** The Qgis Interface to ensure synchronisation*/
    QgisInterface *mIface;

    /** The FingerCount Menu helper*/
    FCTechnique m_fingerCount;

    /** The FingerCount Menu active widget*/
    BlobFingerCount *fcLeft,*fcRight;

    /** The GlassLayer displaying feedbacks*/
    QGisMTGlassLayer *glassLayer;

    /** Track current Vector layer type */
    QGis::GeometryType currVLayerGeomType;

    /** Store initial hand position*/
    QPoint leftPos,rightPos;

    /** Track current command*/
    MODE_CMD_CTRL mode;

    /** The current active Tool*/
    QGisMTTool* currentTool;

    /** The Pan Tool instance*/
    QGisMTToolPan* toolPan;

    /** The Zoom Tool instance*/
    QGisMTToolZoom* toolZoom;

    /** The Select Tool instance*/
    QGisMTToolSelect* toolSelect;

    /** The Void Tool instance*/
    QGisMTToolVoid* toolVoid;

    /** The Legend Dialog*/
    QDialog* legendDialog;

    /** The Legend widget*/
    QGisMTLegend* my_legend;

    /** The Map Canvas containing the rendered map*/
    QgsMapCanvas* my_canvas;

    /** List of the layers availlable in the map canvas*/
    QList<QgsMapCanvasLayer> canvasLayer;


public slots:

    /** Set the current Action Mode*/
    virtual void setMode ( MODE_CMD_CTRL _mode )
    {
        mode = _mode;
    }
    
    /** Store the Hand position at the beginning of the action*/
    virtual void initHandPosition ( const QLine& l1, const QLine& l2 );
    
    /** Activate the Tools*/
    virtual void commandControl ( const QLine& l1, const QLine& l2 );
    
    /** Set the Active Tool*/
    virtual void updateCommandSelected ( const QString& name );
    
    /** Finnish the tool action and reset the active tool*/
    virtual void updateCommandActivated ( const QString& name );
    
    /** Display the current layer Property*/
    virtual void showLayerProperty();
    
    /** Display the current vector layer attribute table*/
    virtual void showLayerAttrib();
    
    /** Set the active layer by its reference*/
    virtual void setActiveLayer ( QgsMapLayer* layer );
    /** Set the active layer by its key*/
    virtual void setActiveLayer ( QString* layerKey );
    
    /** Zoom on active layer extent*/
    virtual void setZoomLayerExtent();
    
    /** (Des)Activate vector layer edition tools*/
    virtual void toggleLayerEdit();
    
    /** Cancel current vector layer changes and disable edition*/
    virtual void rollbackEdit();
    
    /** Display the legend and permit to choose the current active layer*/
    virtual void chooseLayer();
    
    /** Zoom in*/
    virtual void actionZoomIn();
    
    /** Zoom out*/
    virtual void actionZoomOut();
    
    /** Force synchronisation between QgisMT and Qgis*/
    virtual void openProjectFromQGis();
    
    /** Quit the program*/
    virtual void actionQuit();
    
    /** Invert the selection of the active layer*/
    virtual void invertSelection();
    
    /** Clear the selection of the active layer*/
    virtual void clearSelection();

    /** Zoom to the extent of all layers*/
    virtual void zoomFullExtent();
    
    /** Hide the QGisMT windows */
    virtual void actionReturnQGis();
    
    /** Pan start*/
    virtual void panToolActionStart();
    /** Pan stop*/
    virtual void panToolActionStop();


protected slots:

    /** Initialise FingerCount Menu*/
    virtual void initMenu();
    
    /** Update the current vector layer type*/
    virtual void updateCurrentLayerType ( QgsMapLayer* layer );
    
    /** Update menus to match the current layer type*/
    virtual void updateMenu ( QGis::GeometryType current,QGis::GeometryType previous );

signals:
  
    /** Emited when the current layer type is different from the previous*/
    void geometryTypeChanged ( QGis::GeometryType current,QGis::GeometryType previous );

};

#endif // QGISMTWINDOW_H
